/*
 * Moisture Balancing Module
 * ---------------------------------------------------------
 *
 * Module to regulate the moisture in the soil if critically low
 *
 * Steven Shyaka for Robot Missions
 *
 * Steven Shyaka
 *
 * Apr. 08th, 2018
 */
#include <Streaming.h>
#include <PromulgateBig.h>
#include <StopWatch.h>
#include <Servo.h>

Servo servoMotor1;
Servo servoMotor2;
StopWatch MySW;
Promulgate promulgate;
unsigned long elapsed_time = 0;

char c;
int moisture_percentage_threshold = 10; //The critical moisture point (can be changed here)
int pump_runtime = 15; //The time

void setup() {
  Serial.begin(9600);
  //Servo setup
  servoMotor1.attach(13); //Left servo on the mount (digital pin 13)
  servoMotor2.attach(11); //Right servo on the mount (digital pin 11)
  //Waterpump setup
  pinMode(9, OUTPUT); //Water pump (digital pin 9)
  //promulgate setup
  promulgate = Promulgate(&Serial,&Serial);
  promulgate.LOG_LEVEL = promulgate.ERROR_;
  promulgate.set_debug_stream(&Serial);
  promulgate.set_rx_callback(Moisture_Balancing);
}

void loop() {
  reverse(); //The Moisture Probe stays at rest unless a proper button #6 is pressed twice
  while (Serial.available() > 0){ //If any command from the controller is forwarded to the Arduino
    c = Serial.read();
    promulgate.organize_message(c);//Organizes the message, calls parse_message()
  }                                //and the callback function
}

void Moisture_Balancing(char action, char cmd, uint8_t key, uint16_t val, char cmd2, uint8_t key2, uint16_t val2, char delim){

  int sensorValue; //Variable that holds raw Analog Moisture Probe Values
  int percentagesensorValue;  //Variable that holds the mapped raw Analog Moisture Probe Values
  pump_runtime = pump_runtime*1000 + 7000;
/*---------------------------------------------------------------------------
 * If button #6 is pressed twice
 * Arduino sends a stop command to the Bowie (fail-safe against breaking the moisture probe while inserted)
 * Sensor is inserted into the ground for 6 seconds (6000 milliseconds)
 * If moisture is under threshold moisture
 *  The pump runs for modifiable number of seconds
 *   Module goes back to rest waiting for next controller command
 * If moisture is above threshold moisture
 *  Module goes back to rest waiting for next controller command
 */
  if(cmd == 'N' && key == 1){
    MySW.start();
    Serial.println("#L1,0,R1,0!");
    forward();
    while(elapsed_time < 6000){
      elapsed_time = MySW.elapsed();
      if(elapsed_time < 5000){
        sensorValue = analogRead(A0);
        percentagesensorValue = map(sensorValue, 250, 670, 100, 0);
      }
    }
    if(elapsed_time >= 6000){
      if (percentagesensorValue < moisture_percentage_threshold){
        reverse();
        delay(1000);
        digitalWrite(9, HIGH);
        while(elapsed_time < pump_runtime){
          elapsed_time = MySW.elapsed();
        }
        if (elapsed_time >= pump_runtime){
          pump_off();
        }
      }
      else {
        pump_off();
      }
    }
  }
}

void pump_off() {
  digitalWrite(9,LOW);
  MySW.reset();
  MySW.stop();
  elapsed_time = 0;
}

void forward() {
  servoMotor1.write(160);
  servoMotor2.write(20);
}

void reverse () {
  servoMotor1.write(20);
  servoMotor2.write(160);
}
